<?php
/* --------------------------------------------------------------
   ParcelServiceRepository.inc.php 2018-07-03
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

/**
 * Class ParcelServiceRepository
 */
class ParcelServiceRepository implements ParcelServiceRepositoryInterface
{
	/**
	 * @var \ParcelServiceMapperInterface
	 */
	protected $mapper;
	
	
	/**
	 * ParcelServiceRepository constructor.
	 *
	 * @param \ParcelServiceMapperInterface $mapper
	 */
	public function __construct(\ParcelServiceMapperInterface $mapper)
	{
		$this->mapper = $mapper;
	}
	
	
	/**
	 * Returns a limited collection of parcel services.
	 *
	 * @param \Pager $pager
	 *
	 * @return \ParcelServiceCollection
	 */
	public function findAll(\Pager $pager)
	{
		return $this->mapper->findAll($pager);
	}
	
	
	/**
	 * Returns a limited collection of parcel services found by the given search condition.
	 *
	 * @param \Pager                                 $pager
	 * @param \ParcelServiceSearchCondition $searchCondition
	 *
	 * @return \ParcelServiceCollection
	 */
	public function findBy(\Pager $pager, \ParcelServiceSearchCondition $searchCondition)
	{
		return $this->mapper->findBy($pager, $searchCondition);
	}
	
	
	/**
	 * Returns a parcel services found by its ID.
	 *
	 * @param \ParcelServiceId $parcelServiceId
	 *
	 * @return \ParcelServiceInterface|null
	 */
	public function findById(\ParcelServiceId $parcelServiceId)
	{
		return $this->mapper->findById($parcelServiceId);
	}
	
	
	/**
	 * Saves a given parcel service.
	 *
	 * @param \ParcelServiceInterface $parcelService
	 *
	 * @return \ParcelServiceInterface
	 */
	public function save(\ParcelServiceInterface $parcelService)
	{
		return $this->mapper->insert($parcelService);
	}
	
	
	/**
	 * Updates a given parcel service.
	 *
	 * @param \ParcelServiceInterface $parcelService
	 *
	 * @return \ParcelServiceInterface
	 */
	public function update(\ParcelServiceInterface $parcelService)
	{
		return $this->mapper->update($parcelService);
	}
	
	
	/**
	 * Deletes a given parcel service.
	 *
	 * @param \ParcelServiceInterface $parcelService
	 */
	public function delete(\ParcelServiceInterface $parcelService)
	{
		$this->mapper->delete($parcelService);
	}
}